@@ -60,7 +60,8 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 		self.setItemPrototype(QCell())
 		self.horizontalHeader().hide()
 		self.cellChanged.connect(self._on_cell_changed)
-
+		self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
+		self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
 		# Regular expression to catch all newline varieties of various OS's
 		self.newlines_re = re.compile(r'(\r\n|\r|\n)')
 
@@ -100,14 +101,51 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 
 		self.clear()
 		self._adjust_size()
+		self._last_visible_row = 0
 		for colnr, (name, col) in enumerate(self.dm.columns):
 			self._setcell(0, colnr, name)
-			for rownr, val in enumerate(col):
-				self._setcell(rownr+1, colnr, val)
-			self._optimize_column_width(colnr)
+		self._fill_visible_cells()
 		if self.read_only:
 			self.setColumnCount(len(self.dm.columns))
 			self.setRowCount(len(self.dm) + 1)
+			
+	@silent
+	@fix_cursor
+	@disconnected
+	def verticalScrollbarValueChanged(self, pos):
+		
+		self._fill_visible_cells()
+	
+	@silent
+	@fix_cursor
+	@disconnected
+	def resizeEvent(self, event):
+		
+		self._fill_visible_cells()
+
+	@silent
+	@fix_cursor
+	@disconnected
+	def showEvent(self, event):
+		
+		self._fill_visible_cells()
+			
+	def _fill_visible_cells(self, fill_all=False):
+		
+		if self._last_visible_row == len(self.dm):
+			return
+		# For performance reasons we don't initialize read-tables completely.
+		# This makes it possible to view very larges data structures.
+		last_row = len(self.dm) if fill_all else min(
+			len(self.dm),
+			self.rowAt(self.height())
+		)		
+		for colnr, (name, col) in enumerate(self.dm.columns):
+			for rownr in range(self._last_visible_row, last_row):
+				val = col[rownr]
+				self._setcell(rownr + 1, colnr, val)
+			self._optimize_column_width(colnr)
+		self._last_visible_row = last_row
 
 	# Overridden functions
 
@@ -125,6 +163,24 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 		from qdatamatrix._qcontextmenu import QContextMenu
 		QContextMenu(self).exec_(e.globalPos())
 
+	@silent
+	@fix_cursor
+	@disconnected
+	def selectAll(self):
+		
+		"""
+		desc:
+			Selects all existing cells.
+		"""
+		
+		self._fill_visible_cells(fill_all=True)
+		self.setRangeSelected(
+			QtWidgets.QTableWidgetSelectionRange(
+				0, 0, len(self.dm), len(self.dm.columns) - 1
+			),
+			True
+		)
+
 	# Private functions
 
 	def _adjust_size(self):
@@ -178,7 +234,10 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 	@_cursor_pos.setter
 	def _cursor_pos(self, pos):
 
+		selection_ranges = self.selectedRanges()
 		self.setCurrentCell(pos[0], pos[1])
+		for selection_range in selection_ranges:
+			self.setRangeSelected(selection_range, True)
 
 	def _column_by_index(self, colnr):
 
@@ -280,7 +339,6 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 		if self.read_only:
 			item.setFlags(item.flags() ^ QtCore.Qt.ItemIsEditable)
 
-
 	@property
 	def _unique_name(self):
 
@@ -373,6 +431,7 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 
 		self._copy(clear=True, copy=False)
 
+	@silent
 	def _copy(self, clear=False, copy=True):
 
 		"""
@@ -390,13 +449,13 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 		"""
 
 		# Get the start and end of the selection
-		l = self.selectedRanges()
-		if len(l) == 0:
+		selection = self.selectedRanges()
+		if not selection:
 			return
-		firstrow = min([r.topRow() for r in l])
-		firstcolnr = min([r.leftColumn() for r in l])
-		lastrow = max([r.bottomRow() for r in l])
-		lastcolnr = max([r.rightColumn() for r in l])
+		firstrow = min([r.topRow() for r in selection])
+		firstcolnr = min([r.leftColumn() for r in selection])
+		lastrow = max([r.bottomRow() for r in selection])
+		lastcolnr = max([r.rightColumn() for r in selection])
 		colspan = lastcolnr - firstcolnr + 1
 		rowspan = lastrow - firstrow + 1
 		# Create an empty list of lists, where the value __empty__ indicates
@@ -406,16 +465,24 @@ class QSpreadSheet(QtWidgets.QTableWidget):
 		for col in range(rowspan):
 			matrix.append([EMPTY_STR]*colspan)
 		# Add all selected cells.
+		columns_to_optimize = set()
 		for item in self.selectedItems():
-			row = self.row(item)-firstrow
-			colnr = self.column(item)-firstcolnr
-			matrix[row][colnr] = item.text()
+			rownr = self.row(item) - firstrow
+			if rownr > self._last_visible_row:
+				for colnr, (name, col) in enumerate(self.dm.columns):
+					self._setcell(rownr + 1, colnr, col[rownr])
+					columns_to_optimize.add(colnr)
+			colnr = self.column(item) - firstcolnr
+			matrix[rownr][colnr] = item.text()
 			if clear:
 				self._setcell(self.row(item), self.column(item))
+		for colnr in columns_to_optimize:
+			self._optimize_column_width(colnr)
+		if not copy:
+			return
 		# Convert the selection to text and put it on the clipboard
 		txt = u'\n'.join([u'\t'.join(_col) for _col in matrix])
-		if copy:
-			self._clipboard.setText(txt)
+		self._clipboard.setText(txt)
 
 	@silent
 	@undoable
